#
# Copyright (c) 2008-2018 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

import logging
logger = logging.getLogger('PyAMS (content)')

from pyramid.decorator import reify
from zope.interface import implementer

from pyams_cache.beaker import get_cache
from pyams_content.features.header.interfaces import IHeaderRenderer, IHeaderSettings, IHeaderTarget
from pyams_content.features.header.skin.interfaces import IHeaderClass
from pyams_content.features.renderer.interfaces import HIDDEN_RENDERER_NAME
from pyams_content.features.renderer.skin import BaseContentRenderer
from pyams_portal.interfaces import PREVIEW_MODE
from pyams_portal.portlet import PORTLETS_CACHE_KEY, PORTLETS_CACHE_NAME, PORTLETS_CACHE_REGION
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.adapter import adapter_config
from pyams_utils.interfaces import ICacheKeyValue
from pyams_utils.traversing import get_parent

from pyams_content import _


@implementer(IHeaderRenderer)
class BaseHeaderRenderer(BaseContentRenderer):
    """Base header renderer"""

    name = None
    settings_key = None

    @reify
    def settings_target(self):
        context = self.request.display_context
        if context is None:
            context = self.context
        return get_parent(context, IHeaderTarget)

    @reify
    def settings(self):
        if self.settings_interface is None:
            return None
        target = self.settings_target
        settings = IHeaderSettings(target)
        while settings.inherit:
            settings = IHeaderSettings(settings.parent)
        return settings.settings

    @reify
    def cache_key(self):
        return PORTLETS_CACHE_KEY.format(portlet=self.name,
                                         hostname=self.request.host,
                                         context=ICacheKeyValue(self.settings_target),
                                         lang=self.request.locale_name)

    @property
    def main_header_class(self):
        request = self.request
        return request.registry.queryMultiAdapter((request.context, request), IHeaderClass, default='')

    def render(self):
        preview_mode = self.request.annotations.get(PREVIEW_MODE, False)
        if preview_mode:
            return super(BaseHeaderRenderer, self).render()
        else:
            portlets_cache = get_cache(PORTLETS_CACHE_REGION, PORTLETS_CACHE_NAME)
            cache_key = self.cache_key
            if self.context is not self.request.context:  # display shared content
                cache_key = '{0}::shared'.format(cache_key)
            try:
                result = portlets_cache.get_value(cache_key)
                logger.debug("Retrieving header content from cache key {0}".format(cache_key))
            except KeyError:
                result = super(BaseHeaderRenderer, self).render()
                portlets_cache.set_value(cache_key, result)
                logger.debug("Storing header content to cache key {0}".format(cache_key))
            return result


@adapter_config(name=HIDDEN_RENDERER_NAME, context=(IHeaderTarget, IPyAMSLayer), provides=IHeaderRenderer)
class HiddenHeaderRenderer(BaseHeaderRenderer):
    """Hidden header renderer"""

    name = HIDDEN_RENDERER_NAME
    label = _("Hidden header")
    weight = -999

    def render(self):
        return ''
