#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

from z3c.form import field
from z3c.form.interfaces import DISPLAY_MODE, IDataConverter
from zope.interface import implementer, Interface

from pyams_content import _
# import interfaces
from pyams_content.shared.common.interfaces import IWfSharedContent, IWfSharedContentRoles, IBaseSharedTool
from pyams_content.shared.common.interfaces.types import IWfTypedSharedContent
# import packages
from pyams_content.shared.common.zmi.header import SharedContentHeaderContentProvider
from pyams_form.interfaces.form import IWidgetForm, IInnerTabForm, IInnerSubForm
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.utility import get_principal
from pyams_sequence.interfaces import ISequentialIdInfo
from pyams_skin.interfaces import IInnerPage
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.adapter import adapter_config
from pyams_utils.date import format_datetime
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION
from pyams_utils.timezone import tztime
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import contentprovider_config
from pyams_workflow.interfaces import IWorkflowState, IWorkflow, IWorkflowPublicationInfo, IWorkflowStateHistoryItem, \
    IWorkflowVersions
from pyams_zmi.form import AdminDisplayForm, InnerAdminDisplayForm, InnerAdminAddForm


@pagelet_config(name='summary.html', context=IWfSharedContent, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
@implementer(IWidgetForm, IInnerPage)
class SharedContentSummaryForm(AdminDisplayForm):
    """Shared content summary form"""

    legend = _("Display content summary")
    css_class = 'ams-form form-horizontal form-tight'

    fields = field.Fields(Interface)


@contentprovider_config(name='content_header', context=IWfSharedContent, view=SharedContentSummaryForm,
                        layer=IPyAMSLayer)
class SharedContentSummaryFormHeaderProvider(SharedContentHeaderContentProvider):
    """Shared content summary form header provider"""

    @property
    def back_url(self):
        tool = get_parent(self.context, IBaseSharedTool)
        return absolute_url(tool, self.request, 'admin')


@adapter_config(name='dublincore-summary',
                context=(IWfSharedContent, IPyAMSLayer, SharedContentSummaryForm),
                provides=IInnerTabForm)
class SharedContentDublinCoreSummary(InnerAdminDisplayForm):
    """Shared content DublinCore summary"""

    weight = 1
    tab_label = _("Identity card")
    css_class = 'form-tight'

    @property
    def fields(self):
        fields = field.Fields(IWfSharedContent).select('title')
        if IWfTypedSharedContent.providedBy(self.context):
            fields += field.Fields(IWfTypedSharedContent).select('data_type')
        fields += field.Fields(ISequentialIdInfo).select('public_oid')
        return fields


@adapter_config(name='workflow-waiting-state',
                context=(IWfSharedContent, IPyAMSLayer, SharedContentDublinCoreSummary),
                provides=IInnerSubForm)
class SharedContentWorkflowWaitingState(InnerAdminAddForm):
    """Shared content workflow waiting state summary"""

    legend = _("Requested action")
    weight = 10

    mode = DISPLAY_MODE

    def __new__(cls, context, request, form):
        workflow = IWorkflow(context)
        state = IWorkflowState(context)
        if state.state not in workflow.waiting_states:
            return None
        return InnerAdminDisplayForm.__new__(cls)

    fields = field.Fields(IWorkflowState).select('state', 'state_urgency') + \
        field.Fields(IWorkflowStateHistoryItem).select('comment')

    def updateWidgets(self, prefix=None):
        super(SharedContentWorkflowWaitingState, self).updateWidgets(prefix)
        state = IWorkflowState(self.context)
        if 'state' in self.widgets:
            translate = self.request.localizer.translate
            workflow = IWorkflow(self.context)
            self.widgets['state'].value = translate(_("{state} {date} by {principal}")).format(
                state=translate(workflow.get_state_label(state.state)),
                date=format_datetime(state.state_date),
                principal=get_principal(self.request, state.state_principal).title
            )
        if 'state_urgency' in self.widgets:
            converter = IDataConverter(self.widgets['state_urgency'])
            self.widgets['state_urgency'].value = converter.toWidgetValue(state.state_urgency)
        if 'comment' in self.widgets:
            history_item = state.history[-1]
            self.widgets['comment'].label = _("Associated comment")
            self.widgets['comment'].value = history_item.comment


@adapter_config(name='workflow-publication-state',
                context=(IWfSharedContent, IPyAMSLayer, SharedContentDublinCoreSummary),
                provides=IInnerSubForm)
class SharedContentWorkflowPublicationState(InnerAdminDisplayForm):
    """Shared content workflow publication state summary"""

    legend = _("Publication and retire dates")
    weight = 20

    def __new__(cls, context, request, form):
        info = IWorkflowPublicationInfo(context)
        if not info.publication_effective_date:
            return None
        return InnerAdminDisplayForm.__new__(cls)

    fields = field.Fields(IWorkflowPublicationInfo).select('publication_effective_date', 'push_end_date',
                                                           'publication_expiration_date', 'displayed_publication_date')


@adapter_config(name='workflow-version-summary',
                context=(IWfSharedContent, IPyAMSLayer, SharedContentDublinCoreSummary),
                provides=IInnerSubForm)
class SharedContentWorkflowVersionSummary(InnerAdminDisplayForm):
    """Shared content workflow version summary"""

    legend = _("Current version")
    weight = 30

    fields = field.Fields(IWorkflowState).select('version_id', 'state') + \
        field.Fields(IWfSharedContent).select('creation_label') + \
        field.Fields(IWfSharedContentRoles).select('owner') + \
        field.Fields(IWfSharedContent).select('last_update_label') + \
        field.Fields(IWfSharedContent).select('modifiers')

    def updateWidgets(self, prefix=None):
        super(SharedContentWorkflowVersionSummary, self).updateWidgets(prefix)
        if 'version_id' in self.widgets:
            self.widgets['version_id'].label = _("Version")
        if 'state' in self.widgets:
            translate = self.request.localizer.translate
            workflow = IWorkflow(self.context)
            workflow_state = IWorkflowState(self.context)
            history_item = workflow_state.history[-1]
            self.widgets['state'].value = translate(_("{state} since {date}, by {principal}")).format(
                state=translate(workflow.get_state_label(workflow_state.state)),
                date=format_datetime(tztime(history_item.date)),
                principal=get_principal(self.request, history_item.principal).title)


@adapter_config(name='workflow-history-summary',
                context=(IWfSharedContent, IPyAMSLayer, SharedContentDublinCoreSummary),
                provides=IInnerSubForm)
class SharedContentWorkflowHistorySummary(InnerAdminDisplayForm):
    """Shared content workflow history summary"""

    legend = _("Content history")

    fields = field.Fields(IWorkflowPublicationInfo).select('first_publication_date') + \
        field.Fields(IWfSharedContent).select('first_owner')

    weight = 40

    def updateWidgets(self, prefix=None):
        super(SharedContentWorkflowHistorySummary, self).updateWidgets(prefix)
        workflow = IWorkflow(self.context)
        versions = IWorkflowVersions(self.context)
        value = ''
        for version in versions.get_versions():
            state = IWorkflowState(version)
            pub_date = tztime(state.get_first_state_date(workflow.published_states))
            if pub_date is not None:
                value = format_datetime(pub_date)
                break
        self.widgets['first_publication_date'].value = value
